# Import the necessary modules.
import tkinter
import tkinter as tk
import tkinter.messagebox
import pyaudio
import wave
import os
import sys
sys.path.append('../../')
from datetime import datetime
import keyboard

class RecAUD_space:
    """
    Record audio interface.
    Options to start, stop and cancel recording
    It saves to file automatically when you stop recording.

    Operates by using the keyboard as opposed to pressing buttons
    """

    def __init__(self, file_name, chunk=3024, frmat=pyaudio.paInt16, channels=2, rate=44100, py=pyaudio.PyAudio()):

        # Start Tkinter and set Title

        # set to hit space to record and stop record
        # esc to cancel?
        # output table number, order, time ordered

        # slide on what it's doing with confidence scores etc.

        self.main = tkinter.Tk()
        self.collections = []
        self.main.geometry('300x150') # size of buttons
        self.main.title('Record') # title of interface
        self.CHUNK = chunk
        self.FORMAT = frmat
        self.CHANNELS = channels
        self.RATE = rate
        self.p = py
        self.frames = []
        self.st = 1 # start counter
        self.stream = self.p.open(format=self.FORMAT, channels=self.CHANNELS, rate=self.RATE, input=True, frames_per_buffer=self.CHUNK)
        current_time = str(datetime.now())
        current_time = "_".join(current_time.split()).replace(":","-")
        self.current_time = current_time[:-7]
        self.file_name = file_name

        self.labelframe = tkinter.LabelFrame(self.main, text = "Could I take your order?")
        self.labelframe.unbind("<space>")
        self.labelframe.bind("<space>", self.start_record)
        
        #self.labelframe.bind("<space>", self.stop)
        self.labelframe.bind("<Escape>", self.cancel_record)
        self.labelframe.pack(fill=tk.BOTH, expand = "yes")
        self.label = tkinter.Label(self.labelframe
        , text = "Press space bar to record, and again to stop.\n Press ESC to cancel.")
        self.label.pack()
        self.labelframe.focus_set()
        
        # Set Frames
        self.labelframe.mainloop()

    def start_record(self, event):
        """
        Start recording audio command
        """
        self.st = 1
        self.frames = []
        stream = self.p.open(format=self.FORMAT, channels=self.CHANNELS, rate=self.RATE, input=True, frames_per_buffer=self.CHUNK)
        #print("recording your order")
        while self.st == 1:
            data = stream.read(self.CHUNK)
            print("recording your order")
            self.frames.append(data)
            self.main.update()
            if keyboard.is_pressed('space'):
                #self.labelframe.unbind("<space>")
                self.st = 0
                stream.close()

                wf = wave.open(self.file_name, 'wb')
                wf.setnchannels(self.CHANNELS)
                wf.setsampwidth(self.p.get_sample_size(self.FORMAT))
                wf.setframerate(self.RATE)
                wf.writeframes(b''.join(self.frames))
                wf.close()
                print('Written to file' , self.file_name, 'at time:' + self.current_time)
                self.main.destroy()

        # stream.close()

        # wf = wave.open(self.file_name, 'wb')
        # wf.setnchannels(self.CHANNELS)
        # wf.setsampwidth(self.p.get_sample_size(self.FORMAT))
        # wf.setframerate(self.RATE)
        # wf.writeframes(b''.join(self.frames))
        # wf.close()
        # print('Written to file' , self.file_name, 'at time:' + self.current_time)
        # self.main.destroy()
        return None        
    
    def stop(self, event):
        """
        Stop recording audio command
        sets counter to 0
        """
        self.st = 0
        return None 

    def cancel_record(self, event):
        """
        Cancels recording without saving
        Stops execution
        """
        sys.exit("Cancelled Recording")
        return None
    


class RecAUD:
    """
    Record audio interface.
    Options to start, stop and cancel recording
    It saves to file automatically when you stop recording.
    """

    def __init__(self, file_name, chunk=3024, frmat=pyaudio.paInt16, channels=2, rate=44100, py=pyaudio.PyAudio()):

        # Start Tkinter and set Title

        # set to hit space to record and stop record
        # esc to cancel?
        # output table number, order, time ordered

        # slide on what it's doing with confidence scores etc.

        # set to black
        # output time
        # save to csv

        self.main = tkinter.Tk()
        self.collections = []
        self.main.geometry('300x150') # size of buttons
        self.main.title('Record') # title of interface
        self.CHUNK = chunk
        self.FORMAT = frmat
        self.CHANNELS = channels
        self.RATE = rate
        self.p = py
        self.frames = []
        self.st = 1 # start counter
        self.stream = self.p.open(format=self.FORMAT, channels=self.CHANNELS, rate=self.RATE, input=True, frames_per_buffer=self.CHUNK)
        current_time = str(datetime.now())
        current_time = "_".join(current_time.split()).replace(":","-")
        self.current_time = current_time[:-7]
        self.file_name = file_name

        # Set Frames
        self.buttons = tkinter.Frame(self.main, padx=20, pady=20)

        # Pack Frame
        self.buttons.pack(fill=tk.BOTH)

        # Start, Stop and Cancel buttons
        self.strt_rec = tkinter.Button(self.buttons, width=10, padx=10, pady=5,
         text='Start Recording', command=lambda: self.start_record())
        
        self.strt_rec.grid(row=0, column=0, padx=5, pady=5)
        self.stop_rec = tkinter.Button(self.buttons, width=10, padx=10, pady=5,
         text='Stop Recording', command=lambda: self.stop())
        self.stop_rec.grid(row=0, column=1, columnspan=1, padx=5, pady=5)
        self.cancel_rec = tkinter.Button(self.buttons, width=20, padx=0, pady=5,
         text='Cancel', command=lambda: self.cancel_record())
        self.cancel_rec.grid(row=1, column=0, columnspan=2, padx=50, pady=5)

        tkinter.mainloop()

    def start_record(self):
        """
        Start recording audio command
        """
        self.st = 1
        self.frames = []
        stream = self.p.open(format=self.FORMAT, channels=self.CHANNELS, rate=self.RATE, input=True, frames_per_buffer=self.CHUNK)
        while self.st == 1:
            data = stream.read(self.CHUNK)
            self.frames.append(data)
            print("* recording")
            self.main.update()

        stream.close()

        wf = wave.open(self.file_name, 'wb')
        wf.setnchannels(self.CHANNELS)
        wf.setsampwidth(self.p.get_sample_size(self.FORMAT))
        wf.setframerate(self.RATE)
        wf.writeframes(b''.join(self.frames))
        wf.close()
        print('Written to file' , self.file_name, 'at time:' + self.current_time)
        self.main.destroy()
        return None        
    
    def stop(self):
        """
        Stop recording audio command
        sets counter to 0
        """
        self.st = 0
        return None 

    def cancel_record(self):
        """
        Cancels recording without saving
        Stops execution
        """
        sys.exit("Cancelled Recording")
        return None




if __name__ == "__main__":
    # Create an object of the ProgramGUI class to begin the program.
    guiAUD = RecAUD_space('audio.wav')


import qdarkstyle
import sys
from PySide2 import QtWidgets as qtw
from PySide2 import QtGui as qtg
from PySide2 import QtCore as qtc
from PySide2 import QtMultimedia as qtmm

sys.path.append('../../')
from src.configs.config import filenames
from datetime import datetime
import time

audiofile2 = filenames.audio_file2

class MainWindow(qtw.QMainWindow):
    closed = qtc.Signal()

    def __init__(self):
        """
        MainWindow constructor:
        Code in this method should define window properties, create backend resources, etc.
        """
        super().__init__()
        self.setWindowTitle("Record Order")

        #Create Window layout with a sound widget
        soundboard = qtw.QWidget()
        soundboard.setLayout(qtw.QGridLayout())
        self.setCentralWidget(soundboard)
        sw = SoundWidget()
        soundboard.layout().addWidget(sw)

        # Status Bar
        self.status = self.statusBar()
        self.status.showMessage("AI Waiter")

        #Window Dimensions
        self.setSizePolicy(qtw.QSizePolicy.Expanding, qtw.QSizePolicy.MinimumExpanding)

        # Code ends here
        #self.show()

        # @qtc.Slot()
        # def exit_app(self, checked):
        #     sys.exit()

        sw.sendorder_button.clicked.connect(self.close)

    def closeEvent(self, event):
        self.closed.emit()
        super().closeEvent(event)

class PlayButton(qtw.QPushButton):
    play_stylesheet = 'color: white;'
    stop_stylesheet = 'background-color: darkred; color: white;'

    def __init__(self):
        super().__init__('Play')
        self.setStyleSheet(self.play_stylesheet)

    def on_state_changed(self, state):
        if state == qtmm.QMediaPlayer.PlayingState:
            self.setStyleSheet(self.stop_stylesheet)
            self.setText('Stop')
        else:
            self.setStyleSheet(self.play_stylesheet)
            self.setText('Play')


class RecordButton(qtw.QPushButton):
    record_stylesheet = 'background-color: green; color: white;'
    stop_stylesheet = 'background-color: darkred; color: white;'

    def __init__(self):
        super().__init__('Record')
        self.setFont(qtg.QFont('Sans', 14, qtg.QFont.Bold))
        self.setStyleSheet(self.record_stylesheet)
        self.setSizePolicy(qtw.QSizePolicy.Expanding, qtw.QSizePolicy.Expanding)

    def on_state_changed(self, state):
        if state == qtmm.QAudioRecorder.RecordingState:
            self.setStyleSheet(self.stop_stylesheet)
            self.setText('Stop')
        else:
            self.setStyleSheet(self.record_stylesheet)
            self.setText('Record')

class SendOrderButton(qtw.QPushButton):
    button_stylesheet = 'background-color: blue; color: white;'

    def __init__(self):
        super().__init__('Send Order')
        self.setFont(qtg.QFont('Sans', 14, qtg.QFont.Bold))
        self.setSizePolicy(qtw.QSizePolicy.Expanding, qtw.QSizePolicy.Expanding)
        self.setStyleSheet(self.button_stylesheet)
        #self.clicked.connect(self.close)

    def press_button(self):
        if self.isEnabled():
            self.setEnabled(False)
            self.setText('Send Order')
        else:
            self.setEnabled(True)
            self.setText('Sent')

class SoundWidget(qtw.QWidget):
    def __init__(self):
        super().__init__()
        self.setLayout(qtw.QGridLayout())

        # Title
        self.label = qtw.QLabel("May I take your order?")
        self.label.setFont(qtg.QFont('Sans', 18, qtg.QFont.Bold))
        self.layout().addWidget(self.label, 0, 0, 1, 2)

        # Playback Subtitle
        self.label = qtw.QLabel("No Recording yet")
        self.layout().addWidget(self.label, 1, 0, 1, 2)

        #Play Button
        self.play_button = PlayButton()
        self.layout().addWidget(self.play_button, 3, 1, 1, 1)

        self.player = qtmm.QMediaPlayer()
        self.play_button.clicked.connect(self.on_playbutton)
        self.player.stateChanged.connect(self.play_button.on_state_changed)

        # Loading files Button
        self.file_button = qtw.QPushButton('Load File', clicked=self.get_file)
        self.layout().addWidget(self.file_button, 3, 0, 1, 1)

        # Slider
        self.position = qtw.QSlider(minimum=0, orientation=qtc.Qt.Horizontal)
        self.layout().addWidget(self.position, 2, 0, 1, 2)

        self.player.positionChanged.connect(self.position.setSliderPosition)
        self.player.durationChanged.connect(self.position.setMaximum)
        self.position.sliderMoved.connect(self.player.setPosition)

        # Volume
        # self.volume = qtw.QSlider(minimum=0,maximum=100,sliderPosition=10,
        #     orientation=qtc.Qt.Horizontal,
        #     #sliderMoved=self.player.setVolume
        # )
        # self.layout().addWidget(self.volume, 2, 0)

        # Recording
        self.recorder = qtmm.QAudioRecorder()

        # supported audio inputs
        print(self.recorder.audioInputs())
        self.recorder.setAudioInput('default:')

        #Overriding sound recording settings
        settings = qtmm.QAudioEncoderSettings()
        settings.setCodec('audio/pcm')
        settings.setSampleRate(44100)
        settings.setQuality(qtmm.QMultimedia.HighQuality)
        self.recorder.setEncodingSettings(settings)
        self.recorder.setContainerFormat('audio/x-wav')
        self.recorder.setOutputLocation(qtc.QUrl.fromLocalFile(audiofile2))

        #Record Button
        self.record_button = RecordButton()
        self.layout().addWidget(self.record_button, 4, 0, 1, 2)
        self.recorder.stateChanged.connect(self.record_button.on_state_changed)
        self.record_button.clicked.connect(self.on_recordbutton)
        self.shortcut = qtw.QShortcut(qtg.QKeySequence("Space"), self)
        self.shortcut.activated.connect(self.on_recordbutton)

        #Send Order Button
        self.sendorder_button = SendOrderButton()
        self.sendorder_button.setShortcut(qtg.QKeySequence('Tab'))
        self.layout().addWidget(self.sendorder_button, 5, 0, 1, 2)
        #self.sendorder_button.clicked.connect(qtc.QCoreApplication.instance().quit)
        #self.sendorder_button.clicked.connect(qtc.QCoreApplication.exit(0))

    def on_playbutton(self):
        if self.player.state() == qtmm.QMediaPlayer.PlayingState:
            self.player.stop()
        else:
            self.player.play()

    def set_file(self, url):
        self.label.setText(url.fileName())
        if url.scheme() == '':
            url.setScheme('file')
        content = qtmm.QMediaContent(url)
        self.player.setMedia(content)

    def get_file(self):
        fn, _ = qtw.QFileDialog.getOpenFileUrl(
            self,
            "Select File",
            qtc.QDir.homePath(),
            "Audio files (*.wav *.flac *.mp3 *.ogg *.aiff);; All files (*)"
        )
        if fn:
            self.set_file(fn)

    def on_recordbutton(self):
        if self.recorder.state() == qtmm.QMediaRecorder.RecordingState:
            self.recorder.stop()
            url = self.recorder.actualLocation()
            self.set_file(url)
        else:
            self.recorder.record()

def main():
    #app = qtw.QApplication(sys.argv)
    window = MainWindow()
    timestamp = time.strftime('%H:%M:%S')
    #app.setStyleSheet(qdarkstyle.load_stylesheet_pyside2())
    #window.show()
    #app.exec_()
    return window, timestamp

if __name__ == '__main__':
    import sys

    app = qtw.QApplication(sys.argv)
    window = MainWindow()
    app.setStyleSheet(qdarkstyle.load_stylesheet_pyside2())
    window.show()
    sys.exit(app.exec_())


class filenames:
    #code_dir = 'C://Users/KristiaKarakatsani/Projects/AIWaiter/' #'/Users/kristiakarakatsani/Documents/wAIter/' #
    #code_dir = 'C://Users/JosephinePark/Documents/projects/wAIter/'
    audio_dir = 'audio/'
    menu_dir = 'menus/'
    order_dir = 'orders/'
    code_dir = 'src/'

    audio_file = audio_dir + 'order.wav'
    audio_file2 = audio_dir + 'order_numeric.wav'
    menu_file = menu_dir + 'menu.csv'
    gui_photo = 'photo.ico'
    final_orders = audio_dir + 'orders.csv'



import pandas as pd
import sys
import qdarkstyle
import sqlalchemy
import textwrap
from PySide2.QtCore import (QAbstractTableModel, QModelIndex, Slot, QPersistentModelIndex, Qt)
from PySide2.QtWidgets import (QDesktopWidget, QAction, QApplication, QHBoxLayout, QHeaderView, QMainWindow,
                               QSizePolicy, QTableView, QWidget, QCheckBox, QTableWidgetItem)
from PySide2.QtGui import QFont
sys.path.append('../')
from configs.config import filenames

final_orders = filenames.final_orders

def read_sql_server():
    engine = sqlalchemy.create_engine(
        "mssql+pyodbc://altius:alt.ius01@demodatasets.database.windows.net/DemoDatasets?driver=SQL+Server")
    query = textwrap.dedent('''
                SELECT *
                FROM orders''')
    df = pd.read_sql_query(query, engine)
    order = df["OrderID"]
    itemid = df["ItemID"]
    table = df["TableNo"]
    item = df["Item"]
    quantity = df["Quantity"]
    number = df["Number"]
    time = df["Time"]
    return order, itemid,table, item, quantity, number, time

# def read_data(fname):
#     df = pd.read_csv(fname)
#     table = df["Table"]
#     order = df["Order"]
#     time = df["Time"]
#     return table, order, time

class MainWindow(QMainWindow):
    def __init__(self, widget):
        super().__init__()
        #QMainWindow.__init__(self)
        self.setWindowTitle("AI Waiter Orders")

        # Menu
        self.menu = self.menuBar()
        self.file_menu = self.menu.addMenu("File")

        # Exit QAction
        exit_action = QAction("Exit", self)
        exit_action.setShortcut("Ctrl+Q")
        exit_action.triggered.connect(self.close)

        self.file_menu.addAction(exit_action)

        # Status Bar
        self.status = self.statusBar()
        self.status.showMessage("Orders")

        # Window dimensions
        # geometry = QDesktopWidget().availableGeometry(self)
        # self.setFixedSize(geometry.width() * 1, geometry.height() * 1)
        self.setGeometry(0, 0, 1550, 800)
        self.setCentralWidget(widget)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        #Font
        self.setFont(QFont('SansSerif', 14))

    # @Slot()
    # def exit_app(self, checked):
    #     sys.exit()

class CustomTableModel(QAbstractTableModel):
    def __init__(self, data=None):
        QAbstractTableModel.__init__(self)
        self.load_data(data)
        self.checks = {}

    def checkState(self, index):
        if index in self.checks.keys():
            return self.checks[index]
        else:
            return Qt.Unchecked

    def load_data(self, data):
        self.input_order = data[0].values
        self.input_itemid = data[1].values
        self.input_table = data[2].values
        self.input_item = data[3].values
        self.input_quantity = data[4].values
        self.input_number = data[5].values
        self.input_time = data[6].values

        self.column_count = 8
        self.row_count = len(self.input_order)

    def rowCount(self, parent=QModelIndex()):
        return self.row_count

    def columnCount(self, parent=QModelIndex()):
        return self.column_count

    def headerData(self, section, orientation, role):
        if role != Qt.DisplayRole:
            return None
        if orientation == Qt.Horizontal:
            return ("Served", "OrderID", "ItemID", "TableNo", "Item", "Quantity", "Number", "Time")[section]
        else:
            return "{}".format(section)

    def data(self, index, role = Qt.DisplayRole):
        column = index.column()
        row = index.row()
        if role == Qt.DisplayRole or role == Qt.EditRole:
            if column == 1:
                raw_date = self.input_order[row]
                date = "{}".format(raw_date)
                return date
            elif column == 2:
                return "{}".format(self.input_itemid[row])
            elif column == 3:
                return "{}".format(self.input_table[row])
            elif column == 4:
                return "{}".format(self.input_item[row])
            elif column == 5:
                return "{}".format(self.input_quantity[row])
            elif column == 6:
                return "{}".format(self.input_number[row])
            elif column == 7:
                return "{}".format(self.input_time[row])
            else:
                return QCheckBox('')
        elif role == Qt.CheckStateRole and column == 0:
            return self.checkState(QPersistentModelIndex(index))
        elif role == Qt.TextAlignmentRole:
            return Qt.AlignCenter

        return None

    def setData(self, index, value, role=Qt.EditRole):
        if not index.isValid():
            return False
        if role == Qt.CheckStateRole:
            self.checks[QPersistentModelIndex(index)] = value
            return True
        return False

    def flags(self, index):
        fl = QAbstractTableModel.flags(self, index)
        if index.column() == 0:
            fl |= Qt.ItemIsEditable | Qt.ItemIsUserCheckable
        return fl

class Widget(QWidget):
    def __init__(self, data):
        super().__init__()
        #QWidget.__init__(self)

        # Getting the Model
        self.model = CustomTableModel(data)

        # Creating a QTableView
        self.table_view = QTableView()
        self.table_view.setModel(self.model)

        # QTableView Headers
        self.horizontal_header = self.table_view.horizontalHeader()
        self.vertical_header = self.table_view.verticalHeader()
        self.horizontal_header.setSectionResizeMode(0,QHeaderView.ResizeToContents)
        self.horizontal_header.setSectionResizeMode(1,QHeaderView.ResizeToContents)
        self.horizontal_header.setSectionResizeMode(2,QHeaderView.ResizeToContents)
        self.horizontal_header.setSectionResizeMode(3,QHeaderView.ResizeToContents)
        self.horizontal_header.setSectionResizeMode(4,QHeaderView.Stretch)
        self.horizontal_header.setSectionResizeMode(5,QHeaderView.ResizeToContents)
        self.horizontal_header.setSectionResizeMode(6,QHeaderView.ResizeToContents)
        self.horizontal_header.setSectionResizeMode(7,QHeaderView.ResizeToContents)
        self.vertical_header.setSectionResizeMode(QHeaderView.Interactive)
        self.horizontal_header.setStyleSheet("QHeaderView { font-size: 18pt; font: bold 24px;}")

        # QWidget Layout
        self.main_layout = QHBoxLayout()
        size = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        self.main_layout.setContentsMargins(0,0,0,0)

        #Align Checkbox
        self.main_layout.setContentsMargins(0,0,0,0)

        ## Left layout
        size.setHorizontalStretch(1)
        self.table_view.setSizePolicy(size)
        self.main_layout.addWidget(self.table_view)

        # Set the layout to the QWidget
        self.setLayout(self.main_layout)

def main(final_orders):
    data = read_sql_server()
    #app2 = QApplication()
    widget = Widget(data)
    window2 = MainWindow(widget)
    #app.setStyleSheet(qdarkstyle.load_stylesheet_pyside2())
    #window2.show()
    #sys.exit(app2.exec_())
    return window2

if __name__ == "__main__":
    data = read_sql_server()

    # Qt Application
    app = QApplication()
    # QWidget
    widget = Widget(data)
    # QMainWindow using QWidget as central widget
    window = MainWindow(widget)
    # setup stylesheet
    app.setStyleSheet(qdarkstyle.load_stylesheet_pyside2())

    window.show()
    sys.exit(app.exec_())


""" 
test space bar to record audio
"""

import tkinter
import keyboard
import sys
import pyaudio

# def greet(event):
#     print('hello')

# def farewell(event):
#     print('bye')

# root = tkinter.Tk()
# frame = tkinter.Frame(root, width = 100, height = 100)
# frame.bind("<space>", greet)
# frame.bind("<Return>", farewell)
# frame.pack()
# frame.focus_set()
# frame.mainloop()

def start(event):
    """
    Start recording audio command
    """
    st = 1
    #stream = p.open(format=FORMAT, channels=CHANNELS, rate=RATE, input=True, frames_per_buffer=CHUNK)
    while st == 1:
        print("hello")
        if keyboard.is_pressed('space'):
            st = 0
            sys.exit("Stopped")



def cancel(event):
    sys.exit("Cancelled recording")

def record(file_name, chunk=3024, frmat=pyaudio.paInt16, channels=2, rate=44100, py=pyaudio.PyAudio()):


    main = tkinter.Tk()

    labelframe = tkinter.LabelFrame(main, text = "Could I take your order?")
    labelframe.pack(fill=tkinter.BOTH, expand = "yes")
    labelframe.bind("<space>", start)
    labelframe.bind("<Escape>", cancel)
    label = tkinter.Label(labelframe
    , text = "Press space bar to record, and again to stop.\n Press ESC to cancel.")
    label.pack()
    labelframe.focus_set()
    labelframe.mainloop()
    
record("hello")



import sys
sys.path.append('../')
#from src.convert_speech.record_audio import RecAUD, RecAUD_space
import src.convert_speech.record_audio2 as record_audio
from src.convert_speech.speech_to_text import SpeechConvert
from src.configs.config import filenames
import src.convert_text.get_menu_items as get_menu
import src.convert_text.text_to_command as get_command
import speech_recognition as sr
import src.gui.display_window2 as display_qt
from PySide2 import QtWidgets as qtw
import qdarkstyle
import os
os.chdir('..')


#import text_to_command
def main(audio_file, menu_file, final_orders):

    app = qtw.QApplication(sys.argv)
    app.setStyleSheet(qdarkstyle.load_stylesheet_pyside2())
    w1, timestamp =record_audio.main()
    print(timestamp)
    w1.show()
    app.exec_()
    w1.closed

    #RecAUD(audio_file)
    r = sr.Recognizer()
    audio = SpeechConvert(audio_file, r)
    menu = get_menu.main(menu_file)
    get_command.main(audio, timestamp, r, menu)

    #sys.exit(app.exec_())

if __name__ == "__main__":
    audio_file = filenames.audio_file2
    #r = sr.Recognizer()
    #audio = SpeechConvert(audio_file, r)
    menu_file = filenames.menu_file
    gui_photo = filenames.gui_photo
    final_orders = filenames.final_orders
    main(audio_file, menu_file, final_orders)




from nltk import word_tokenize
from statistics import mode

import sys
sys.path.append('../../')
from src.convert_text.get_menu_items import lemmatize_text, clean_text, remove_stopwords
from src.configs.config import filenames

import textwrap
import json
import ast
from collections import Counter
import pandas as pd
import random
import nltk
import sqlalchemy
import random
from datetime import datetime
import urllib3, time
import requests
import json
from random import randint
#from word2number import w2n

import sqlalchemy


def clean_speech(text):
    """
    Cleans the speech to text conversione

    Args:
        text : string of text converted from speech
    
    Returns:
        new_text : list of strings
    """

    new_text = clean_text(text)
    return lemmatize_text(new_text)

def search_for_general_item(text, menu):
    """
    search the cleaned text for an item on the menu

    Args:
        text : speech converted into text
        menu : menu data from csv file

    Return:
        item_idx (int): possible indices of menu item
        x : corresponding menu item
    """

    # is the general term in the order?
    general_terms = menu['General term'].unique()
    
    possible_items = []
    for w in text:
       possible_items = possible_items + [x for x in general_terms if w in x]

    # count most frequently occuring item
    if possible_items != []:
        try:
            item = mode(possible_items)
        except:
            print("more than one possible item")
    else:
        item = []

    return item

def search_for_specific_item(text, menu):
    """
    matches the indices for the general item to a specific item

    Args:
        text : speech converted to text
        menu : menu dataframe
        idx : list of possible general items

    TODO: need to improve so it matches up exact word for word

    """


    specific_items = menu['Specific term'].apply(lambda x: remove_stopwords(x))

    print(specific_items)
    possible_items = []
    print('text = ',text)
    for w in text:
        possible_items += [x for x in specific_items if w in x]
    
    if possible_items == []:
        print('no match to specific item found')
        return []
    else:
        try:
            return mode(possible_items)
        except:
            print("More than one possible item")
            print(possible_items)
            return []

def match_item_to_order(text, menu):
    """
    instead of seraching for an item in the text, search through all the items
    to see if they appear in the text
    """

    # clean specific items
    menu['Specific term'] = menu['Specific term'].apply(lambda x: remove_stopwords(x))
    text = word_tokenize(remove_stopwords(' '.join(text)))
    possible_items = []
    count_terms = []
    for term in menu['Specific term']:
        count = 0
        for token in word_tokenize(term):
            if any(token == w for w in text):
                count +=1
        if count != 0:
            possible_items.append(term)
            count_terms.append(count)

    if possible_items == []:
        return []
    else:
        try:
            return mode(possible_items)
        except:
            print('count_terms = ',count_terms)
            print('possible_items = ', possible_items)
            idx = count_terms.index(max(count_terms))
            for i in range(len(count_terms)):
                if i != idx:
                    if count_terms[i] == max(count_terms):
                        return 'not sure'
            return possible_items[idx]
            

def search_for_quantity(text,menu, item_index):

    # TODO : find numeric values
    quantity_list = menu['Quantities'].loc[item_index]
    quantity = ([w for w in text if w in quantity_list])

    return quantity

def search_for_number(text):

    pos = nltk.pos_tag(text)
    for i in range(len(pos)):
        word, pos_tag = pos[i]
        if pos_tag == 'CD':
            return word    
    else:
        return 'one'

def get_alternative_text(audio, recognizer):

    alt_dict = recognizer.recognize_google(audio, show_all = True)
    alternatives = alt_dict['alternative']
    transcript = []
    for i in alternatives:
        transcript.append(i['transcript'])
    return transcript

def final_order(order_data, timestamp):
    """
    determine final order, as most likely translation
    """
    max_items = max(order_data['sentence_id'])
    order_itemid = []
    order_table = []
    order_items = []
    order_quantity = []
    order_timestamp = []
    order_number = []

    # iterate through item orders
    for i in range(max_items + 1):

        # find item
        item_list = order_data[order_data['sentence_id'] == i]['item'].values
        # if all items are the same, choice is obvious
        if all(x == item_list[0] for x in item_list):
            item = item_list[0]
        else:
            # TODO : find most commonly occuring item
            try:
                item = mode(item_list)
                if item == 'not sure':
                        item = mode(item_list[item_list != 'not sure'])
            except:
                item = item_list[0]
                
        
        # find quantity
        quantity_list = order_data[order_data['sentence_id'] == i]['quantity'].values
        try:
            quantity_list = [q[0] for q in quantity_list]
            if all(x == quantity_list[0] for x in quantity_list):
                quantity = quantity_list[0]     
            else:
                try:
                    quantity = mode(quantity_list)
                except:
                    quantity = quantity_list[0]
        except:
            quantity = 'missing quantity'


        # find number
        number_list = order_data[order_data['sentence_id'] == i]['number'].values
        try:
            if all(x == number_list[0] for x in number_list):
                number = number_list[0]
            else:
                number = float('NaN')
        except:
            number = float('NaN')

        order_itemid.append(random.randint(1000,2000))
        order_items.append(item)
        order_quantity.append(quantity)
        order_timestamp.append(timestamp)
        order_table.append(0)
        order_number.append(number)

        # TODO : what if not all items/quantites are the same? How to pick most likely order

    #Counts how many times the script has ran to give a unique item ID
    def get_var_value(filename="varstore.dat"):
        with open(filename, "a+") as f:
            f.seek(0)
            val = int(f.read() or 0) + 1
            f.seek(0)
            f.truncate()
            f.write(str(val))
            return val

    your_counter = get_var_value()
    
    final_order_dict = {'OrderID': your_counter
                        ,'TableNo': random.randint(1,25)
                        ,'ItemID': order_itemid
                        ,'Item': order_items
                        ,'Quantity': order_quantity
                        ,'Number': order_number
                        ,'Time': order_timestamp}
    final_order_data = pd.DataFrame(final_order_dict)
    print(final_order_data)
    return final_order_data

def main(audio, timestamp, recognizer, menu):
    """
    Args:
        audio : output from SpeechConvert()
        recognizer : speech recognition class
        menu : dataframe containing menu information

    returns
        command
    """

    # get text and clean - look for alternatives
    speech_list = get_alternative_text(audio, recognizer)
    #speech_text = 'could i get a large glass of pinot grigio and a small glass of sauvignon blanc'

    

    order_list = []
    sentence_list = []
    item_list = []
    order_id = []
    sentence_id = []
    quantity_list = []
    number_list = []
    for i, speech_text in enumerate(speech_list):
    # split order into sentences with the word 'and'
        print('whole order = ',speech_text)
        print("")
        order = []
        speech = speech_text.split('and')
        
        for j, sentence in enumerate(speech):

            print('sentence =', sentence)
            new_text = clean_speech(sentence)
            new_text = remove_stopwords(new_text)
            if len(new_text) == 1:
                print("only one word found")
                sys.exit()
            
            new_text = word_tokenize(new_text)

            # try and find specific item
            item_name = match_item_to_order(new_text, menu)
            

            # if specific item found, search for quantity
            if item_name != []:
                try:
                    item_index = menu.loc[menu['Specific term'] == item_name].index[0] # if multiple items
                    quantity = search_for_quantity(new_text, menu, item_index)
                except:
                    quantity = 'item not clear'
            else:
            # if specific item not found perhaps ordering a general item?
                general_item = search_for_general_item(new_text, menu)

            number = search_for_number(new_text)
            
            

            print('Item: ', item_name, '. Quantity: ', quantity)
            print("")
            order.append([item_name, quantity])

            order_list.append(speech_text)
            sentence_list.append(sentence)
            item_list.append(item_name)
            sentence_id.append(j)
            order_id.append(i)
            quantity_list.append(quantity)
            number_list.append(number)


        print("--------------------")
        if order != []:
            exit

    # save order to file

    save_dir = filenames.order_dir
    filename = 'full_order.csv'
    dct = {'translation_id' : order_id
            ,'sentence_id' : sentence_id
            ,'order' : order_list
            ,'sentence' : sentence_list
            ,'item' : item_list
            ,'quantity': quantity_list
            ,'number' : number_list}
    full_order_data = pd.DataFrame(dct)
    full_order_data.to_csv(save_dir + filename)
    order = final_order(full_order_data, timestamp)
    order.to_csv(save_dir + 'final_order.csv', index = False)

    ###########################
    #Save order to SQL table
    engine = sqlalchemy.create_engine(
        "mssql+pyodbc://altius:alt.ius01@demodatasets.database.windows.net/DemoDatasets?driver=SQL+Server")
    order.to_sql(name='orders', con=engine, if_exists='append', index=False)
    print('order saved')

    return None



"""
Code that convert speech to text
"""


import speech_recognition as sr

def SpeechConvert(file_name, recognizer):
    audio_file = sr.AudioFile(file_name)
    with audio_file as source:
        recognizer.adjust_for_ambient_noise(source, duration = .1)
        audio = recognizer.record(source)

    try:
        print(recognizer.recognize_google(audio))
    except sr.UnknownValueError:
        print("I'm sorry, I cannot understand you. Please try again.")
        sys.exit()
        return
    except sr.RequestError:
        print("Could not request results")
        sys.exit()
        return
        
    return audio


    


from setuptools import setup, find_packages

with open('DESCRIPTION.txt') as file:
    long_description = file.read()

REQUIREMENTS = [
'asn1crypto==1.2.0',
'certifi==2019.11.28',
'cffi==1.13.2',
'chardet==3.0.4',
'Click==7.0',
'cryptography==2.8',
'Flask==1.1.1',
'Flask-SQLAlchemy==2.4.0',
'idna==2.8',
'itsdangerous==1.1.0',
'Jinja2==2.10.3',
'keyboard==0.13.4',
'MarkupSafe==1.1.1',
'nltk==3.4.5',
'numpy==1.17.4',
'pandas==0.25.3',
'PyAudio==0.2.11',
'pycparser==2.19',
'pyodbc==4.0.27',
'pyOpenSSL==19.1.0',
'PySocks==1.7.1',
'python-dateutil==2.8.1',
'pytz==2019.3',
'QDarkStyle==2.7',
'requests==2.22.0',
'six==1.13.0',
'SpeechRecognition==3.6.3',
'SQLAlchemy==1.3.12',
'urllib3==1.25.7',
'Werkzeug==0.16.0',
'win-inet-pton==1.1.0',
'wincertstore==0.2'
]

CLASSIFIERS = [
    "Programming Language :: Python 3.7 ",
]

setup(name='AI_waiter', 
      version='1.0.3', 
      description='Code to read record an order and write to a database', 
      long_description=long_description, 
      url='https://github.com/AltiusBI/wAIter', 
      author='Josie Park and ', 
      author_email='josie.park@altiusdata.com', 
      license='MIT', 
      packages= find_packages(), 
      install_requires=REQUIREMENTS,
      python_requires = '<3.7' ,
      ) 


import pandas as pd
import re
from nltk.corpus import stopwords
from nltk import word_tokenize
from nltk.stem import WordNetLemmatizer, LancasterStemmer
"""
TODO: return most likely specific quantity to make sure that
the term with the highest number of tokens is returned.

clean speech text, so that e.g. fritz-kola -> fritz kola
"""




def clean_text(text):   
    text = str(text).lower() #lowercase
    text = re.sub('\'s', 's', text)
    text = re.sub('\’s', 's', text)
    text = re.sub('\W', ' ', text) #remove punctuation
    text = re.sub('\d+','', text) #remove digits 
    text = re.sub('\s+', ' ', text) #remove whitespace
    text = text.strip(' ')
    return text

def remove_stopwords(text):
    stop_words = stopwords.words('english')

    # add words to stopwords
    add_words = ['de', 'la', 'order', 'could']
    stop_words += add_words
    word_tokens = word_tokenize(text)
    filtered = [w for w in word_tokens if not w in stop_words]
    return ' '.join([w for w in filtered])

def lemmatize_text(text):
    # Lemmatizing
    lemmatizer = WordNetLemmatizer()
    lemmatized_output = ' '.join([lemmatizer.lemmatize(w) for w in text.split()])
    
    #Stemming
    #stemmer = LancasterStemmer()
    #stemmed_words = [stemmer.stem(word) for word in text]
    #text = " ".join(stemmed_words)
    
    return lemmatized_output

def get_quantities(menu):
    """
    separate items in quantity columns into a list
    """

    new_quant = []
    for q in menu['Quantities']:
        try:
            new_quant.append(q.split(', '))
        except:
            new_quant.append([])
    menu['Quantities'] = new_quant

    return menu


def main(menu_file):
    """
    menu file is a csv file containing the menu items
    """

    data = pd.read_csv(menu_file, encoding = 'utf-8')
    data.rename(columns={data.columns[1]: "Specific term"}, inplace=True)

    data["Further details"] = data["Further details"].apply(clean_text)
    data = get_quantities(data)
    data['Quantities'] = data['Quantities'].apply(clean_text)
    data["General term"] = data["General term"].apply(clean_text)
    data["Specific term"] = data["Specific term"].apply(clean_text)

    data["description_tokens"] = data["Further details"].apply(word_tokenize)
    #data["quantities_tokens"] = data["Quantities"].apply(word_tokenize)
    data["general_tokens"] = data["General term"].apply(word_tokenize)
    data["specific_tokens"] = data["Specific term"].apply(word_tokenize)

    data["all_tokens"] = data["specific_tokens"] + data["general_tokens"] + data["description_tokens"]

    data.to_csv('menu_data.csv')
    return data

if __name__ == "__main__":
    menu_file = ('../menus/menu.csv')
    tokenized_data = main(menu_file)
    print(tokenized_data.head())